module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::EdgeMethods
    include RepositoriesProxy

    def increment_views(user, learning_object)
      if !edge_exists? "Views", user.rid, learning_object.id
        create_edge "Views", user.rid, learning_object.id
        learning_object.views = learning_object.views + 1
      end
    end

    def liked?(user, learning_object)
      edge_exists? 'Likes', user.rid, learning_object.id
    end

    #
    # Usage:
    #   repository.for(:learning_objects).like current_user, @learning_object
    #
    def like(user, learning_object)
      like_edge_class = 'Likes'

      if liked? user, learning_object
        raise 'The user already likes this object.'
      end

      create_edge like_edge_class, user.rid, learning_object.id
      learning_object.likes = learning_object.likes + 1
    end

    def dislike(user, learning_object)
      like_edge_class = 'Likes'

      if liked? user, learning_object
        destroy_edge like_edge_class, user.rid, learning_object.id
        learning_object.likes = learning_object.likes - 1

        return true
      end

      raise 'The user hasn`t likes this object.'
    end

    # Example:
    #   list = repository.for(:learning_objects).all
    #   list.each do |learning_object|
    #     learning_object.inspect <LearningObject model>
    #   end
    # def all
    #   learning_objects_hash = connection.query "SELECT FROM LearningObject"
    #   build_objects(learning_objects_hash) || []
    # end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def get_by_dspace_id(id_dspace)
      result = select_by_property(odb_class, "id_dspace", id_dspace)
      build_object result.first
    end

    def get_subjects(learning_object)
      result = get_edges_end("IsAbout", "out", learning_object.id)
      subject_repository.build_objects(result)
    end

    def get_attributes(learning_object)
      result = get_edges_end("HasAttr", "out", learning_object.id)
      attribute_repository.build_objects(result)
    end

    def create_relations(learning_object)
      edges = []
      unless learning_object.subjects.empty?
        edges << create_edges_from_array("IsAbout", learning_object.id, learning_object.subjects, true)
      end
      unless learning_object.attributes.empty?
        edges << create_edges_from_array("HasAttr", learning_object.id, learning_object.attributes, true)
      end
      edges.flatten
    end

    def save(learning_object)
      if learning_object.valid?
        connection.command "INSERT INTO LearningObject CONTENT #{learning_object.to_json}"
      end
    end

    def update_property(learning_object, property, value)
      if accepted_properties.include? property
        connection.command "UPDATE LearningObject SET #{property}='#{value}' WHERE @rid = #{learning_object.id}"
      end
    end

    ##
    # To create "index:learningobject_search" on OrientDB, use the following command:
    #   CREATE INDEX learningobject_search
    #   ON LearningObject (name, description)
    #   FULLTEXT ENGINE LUCENE
    #   METADATA {"analyzer":"org.apache.lucene.analysis.br.BrazilianAnalyzer"}
    def search(qry, order = nil)
      # TODO: fix search metadata from dspace
      unless order.nil?
        order = case order
        when "author"
          "metadata.key['dc.contributor.author']"
        when "publicationasc"
          "metadata.key['dc.date.submitted']"
        when "publicationdesc"
          "metadata.key['dc.date.submitted'] DESC"
        when "title"
          "name ASC"
        else
          nil
        end
      end

      query = "SELECT EXPAND(rid) FROM index:learningobject_search WHERE key LUCENE '#{qry}'"
      query = "SELECT * FROM (" + query + ") ORDER BY #{order}" unless order.nil?
      query = "SELECT @rid.asString(), last_modified FROM (" + query + ")"

      learning_objects_hash = connection.query query, limit: 10000

      # return only rids with their modification time
      learning_objects_hash.map do |e|
        {'@rid': e['rid'], 'last_modified': e['last_modified']}
      end
    end

    def build_object(args={})
      LearningObjectBuilder.build_from_orientdb args
    end

    def build_hash
      hash = super
      hash.delete("likes")
      hash.delete("views")
      hash.delete("downloads")
      hash.delete("subjects")
      hash.delete("attributes")
      hash
    end

    private

    def accepted_properties
      ['thumbnail']
    end

    def odb_class
      "LearningObject"
    end

    def count_likes(learning_object)
      get_in_edges_count "Likes", learning_object.id
    end

    def count_views(learning_object)
      get_in_edges_count "Views", learning_object.id
    end

  end
end
